@@ -1,11 +1,12 @@
 # -*- coding: utf-8 -*-
 from __future__ import print_function
 
+import abc
 import sys, os
 import warnings
 import weakref
 
-from clu.constants.consts import λ, φ, NoDefault, pytuple
+from clu.constants.consts import λ, φ, BASEPATH, PROJECT_NAME, NoDefault, pytuple
 from clu.constants.exceptions import ExportError, ExportWarning
 from clu.constants.polyfills import MutableMapping, lru_cache
 
@@ -40,6 +41,27 @@ def doctrim(docstring):
     # Return a single string:
     return '\n'.join(trimmed)
 
+def itermodule(module):
+    """ Get an iterable of `(name, thing)` tuples for all things
+        contained in a given module (although it’ll probably work
+        for classes and instances too – anything `dir()`-able.)
+    """
+    from clu.constants.consts import BUILTINS
+    
+    keys = tuple(key for key in sorted(dir(module)) \
+                      if key not in BUILTINS)
+    values = (getattr(module, key) for key in keys)
+    return zip(keys, values)
+
+def moduleids(module):
+    """ Get a dictionary of `(name, thing)` tuples from a module,
+        indexed by the `id()` value of `thing`
+    """
+    out = {}
+    for key, thing in itermodule(module):
+        out[id(thing)] = (key, thing)
+    return out
+
 def itermoduleids(module):
     """ Internal function to get an iterable of `(name, id(thing))`
         tuples for all things comntained in a given module – q.v.
@@ -52,7 +74,7 @@ def itermoduleids(module):
     ids = (id(getattr(module, key)) for key in keys)
     return zip(keys, ids)
 
-# Q.v. `thingname_search_by_id(…)` function sub.
+# Q.v. `search_by_id(…)` function sub.
 cache = lru_cache(maxsize=128, typed=False)
 
 # This goes against all logic and reason, but it fucking seems
@@ -63,7 +85,7 @@ cache = lru_cache(maxsize=128, typed=False)
 sysmods = lambda: reversed(tuple(frozenset(sys.modules.values())))
 
 @cache
-def thingname_search_by_id(thingID):
+def search_by_id(thingID):
     """ Cached function to find the name of a thing, according
         to what it is called in the context of a module in which
         it resides – searching across all currently imported
@@ -71,9 +93,9 @@ def thingname_search_by_id(thingID):
         `sys.modules.values()` (which is potentially completely
         fucking enormous).
         
-        This function implements `thingname_search(…)` – q.v.
+        This function implements `search_for_name(…)` – q.v.
         the calling function code sub., and is also used in the
-        implementdation of `determine_module(…)`, - also q.v.
+        implementation of `determine_module(…)`, - also q.v.
         the calling function code sub.
         
         Caching courtesy the `functools.lru_cache(…)` decorator.
@@ -91,24 +113,53 @@ def thingname_search_by_id(thingID):
                     return module, key
     return None, None
 
-def thingname_search(thing):
-    """ Attempt to find the name for thing, using the logic from
-        the `thingname(…)` function, applied to all currently
+def search_for_name(thing):
+    """ Attempt to find the name for “thing”, using the logic from
+        the `search_modules(…)` function, applied to all currently
         imported modules, as indicated from the inspection of
         `sys.modules.values()` (which that, as a search space,
         is potentially fucking enormous).
         
         This function may be called by `determine_name(…)`. Its
-        subordinate internal function, `thingname_search_by_id(…)`,
+        subordinate internal function, `search_by_id(…)`,
         uses the LRU cache from `functools`.
     """
-    return thingname_search_by_id(id(thing))[1]
+    return search_by_id(id(thing))[1]
+
+def search_for_module(thing):
+    """ Attempt to find the module containing “thing”, using the
+        logic from the `search_modules(…)` function, applied to
+        all currently imported modules, as indicated from the
+        inspection of `sys.modules.values()` (which that, as a
+        search space, is potentially fucking enormous).
+        
+        This function may be called by `determine_name(…)`. Its
+        subordinate internal function, `search_by_id(…)`,
+        uses the LRU cache from `functools`.
+    """
+    return search_by_id(id(thing))[0]
+
+def search_modules(thing, *modules):
+    """ Find the name of a thing, according to what it is called
+        in the context of a module in which it resides
+    """
+    inquestion = id(thing)
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore")
+        for module in frozenset(modules):
+            for key, value in itermodule(module):
+                if id(value) == inquestion:
+                    return module, key
+    return None, None
 
 def determine_name(thing, name=None, try_repr=False):
     """ Private module function to find a name for a thing. """
     # Shortcut everything if a name was explictly specified:
     if name is not None:
         return name
+    # … or if called without a “thing”:
+    if thing is None:
+        return None
     # Check for telltale function-object attributes:
     code = None
     if hasattr(thing, '__code__'): # Python 3.x
@@ -136,9 +187,9 @@ def determine_name(thing, name=None, try_repr=False):
     # of objects within imported modules -- it is
     # possible (however unlikely) that this’ll ending
     # up returning None:
-    return thingname_search(thing)
+    return search_for_name(thing)
 
-def path_to_dotpath(path):
+def path_to_dotpath(path, relative_to=None):
     """ Convert a file path (e.g. “/yo/dogg/iheard/youlike.py”)
         to a dotpath (á la “yo.dogg.iheard.youlike”) in what I
         would call a “quick and dirty” fashion.
@@ -147,24 +198,30 @@ def path_to_dotpath(path):
         dashes – I don’t quite know what to do about something
         like that besides warn, so erm. There you go.
     """
-    from clu.constants.consts import BASEPATH, QUALIFIER
+    from clu.constants.consts import QUALIFIER
     from clu.constants.exceptions import BadDotpathWarning
     
     # Garbage in, garbage out:
     if path is None:
         return None
     
-    # Relativize the path to the BASEPATH,
+    # Relativize the path to either the “relative_to” arg
+    # …or – if that’s not a thing – the filesystem root,
     # and replace slashes with dots:
-    relpath = os.path.relpath(path, start=BASEPATH)
+    relpath = os.path.relpath(path, start=relative_to or "/")
     dotpath = relpath.replace(os.path.sep, QUALIFIER)
     
     # Trim off any remaining “.py” suffixes,
-    # and extraneous dot-prefixes:
-    if dotpath.endswith('.__init__.py'):
-        dotpath = dotpath[:len(dotpath)-12]
-    elif dotpath.endswith('.py'):
-        dotpath = dotpath[:len(dotpath)-3]
+    # and extraneous dot-prefixes –
+    # N.B. Items in “replaceable_endings” tuple that
+    # possibly contain other such items should appear
+    # *before* the items that they contain, e.g.:
+    replaceable_endings = ('.__init__.py', '.py')
+    
+    for ending in replaceable_endings:
+        if dotpath.endswith(ending):
+            dotpath = dotpath[:len(dotpath)-len(ending)]
+    
     while dotpath.startswith(QUALIFIER):
         dotpath = dotpath[1:]
     
@@ -183,20 +240,165 @@ def predicates_for_types(*types):
         predicates.append(lambda thing: isinstance(thing, classtype))
     return tuple(predicates)
 
-class Exporter(MutableMapping):
+class Slotted(abc.ABCMeta):
     
-    """ A class representing a list of things for a module to export. """
-    __slots__ = pytuple('exports', 'weakref') + ('path', 'dotpath')
-    instances = weakref.WeakValueDictionary()
+    """ A metaclass that ensures its classes, and all subclasses,
+        will be slotted types.
+    """
     
-    def __new__(cls, *args, **kwargs):
-        try:
-            instance = super(Exporter, cls).__new__(cls, *args, **kwargs)
-        except TypeError:
-            instance = super(Exporter, cls).__new__(cls)
+    def __new__(metacls, name, bases, attributes, **kwargs):
+        """ Override for `abc.ABCMeta.__new__(…)` setting up a
+            derived slotted class.
+        """
+        if '__slots__' not in attributes:
+            attributes['__slots__'] = tuple()
         
-        instance.path = kwargs.pop('path', None)
-        instance.dotpath = path_to_dotpath(instance.path)
+        return super(Slotted, metacls).__new__(metacls, name,
+                                                        bases,
+                                                        attributes,
+                                                      **kwargs)
+
+class ValueDescriptor(object):
+    
+    __slots__ = ('value',)
+    
+    def __init__(self, value):
+        self.value = value
+    
+    def __get__(self, *args):
+        return self.value
+    
+    def __repr__(self):
+        from clu.constants.consts import ENCODING
+        return isinstance(self.value, str)   and self.value or \
+               isinstance(self.value, bytes) and self.value.decode(ENCODING) or \
+                                            repr(self.value)
+
+class Prefix(Slotted):
+    
+    """ A metaclass to assign a “prefix” class property,
+        extracted from a “prefix” class keyword, to a new
+        slotted type. 
+    """
+    
+    @classmethod
+    def __prepare__(metacls, name, bases, prefix="/", **kwargs):
+        """ Remove the “prefix” class keyword before calling up """
+        return super(Prefix, metacls).__prepare__(name, bases, **kwargs)
+    
+    def __new__(metacls, name, bases, attributes, prefix="/", **kwargs):
+        """ Override for `Slotted.__new__(…)` setting up a
+            derived slotted class that pulls from a “prefix”
+            with the requisite methods defined for access.
+        """
+        if 'prefix' not in attributes:
+            attributes['prefix'] = ValueDescriptor(prefix)
+        
+        return super(Prefix, metacls).__new__(metacls, name,
+                                                       bases,
+                                                       attributes,
+                                                     **kwargs)
+
+classes = {}
+appnames = set()
+
+class Registry(abc.ABC, metaclass=Slotted):
+    
+    """ A class-registry mixin ancestor type suitable for use
+        in the ExporterBase inheritance chain – it uses the
+        “clu.exporting.Slotted” metaclass and respects the
+        class keywords already in use.
+    """
+    
+    @classmethod
+    def __init_subclass__(cls, appname=None, **kwargs):
+        if appname:
+            appnames.add(appname)
+        else:
+            appname = determine_name(cls)
+        if appname in classes:
+            raise ValueError(f"appname already registered: {appname}")
+        classes[appname] = cls
+        super(Registry, cls).__init_subclass__(**kwargs)
+        cls.instances = weakref.WeakValueDictionary()
+        cls.appname = ValueDescriptor(appname)
+    
+    @staticmethod
+    def all_appnames():
+        """ Return a tuple of all registered appnames """
+        return tuple(sorted(appnames))
+    
+    @staticmethod
+    def for_appname(appname):
+        """ Return a subclass for a registered appname """
+        from clu.typology import isstring
+        if not appname:
+            raise ValueError("appname required")
+        if not isstring(appname):
+            raise TypeError("class registry access by string keys only")
+        return classes[appname]
+    
+    @classmethod
+    def __class_getitem__(cls, key):
+        """ Return a specific registered class from the registry,
+            given an “appname”, like e.g:
+            
+                from clu import exporting
+                assert Registry['clu'] is exporting.Exporter
+        """
+        return cls.for_appname(key)
+    
+    @classmethod
+    def module_getters(cls):
+        from clu.predicates import attr_across
+        return attr_across('modules', *[classes[appname] \
+                           for appname in cls.all_appnames()])
+    
+    @classmethod
+    def all_modules(cls):
+        from itertools import chain
+        return tuple(chain.from_iterable(modules().values() \
+                     for modules in cls.module_getters()))
+    
+    @classmethod
+    def nameof(cls, thing):
+        """ Find and return the name of a thing, if that thing
+            should be found to reside in one of the exported modules
+        """
+        return search_modules(thing, *cls.all_modules())[1]
+    
+    @classmethod
+    def moduleof(cls, thing):
+        """ Find and return the module for a thing, if that thing
+            should be found to reside in one of the exported modules
+        """
+        return search_modules(thing, *cls.all_modules())[0]
+
+class ExporterBase(MutableMapping, Registry, metaclass=Prefix):
+    
+    """ The base class for “clu.exporting.Exporter”. Override this
+        class in your own project to use the CLU exporting mechanism –
+        q.v. “clu.exporting.Exporter” docstring sub.
+        
+        This class uses the “clu.exporting.Prefix” metaclass, which
+        automatically adds a “prefix” class attribute, as well as
+        the “clu.exporting.Registry” mixin, which keeps a registry
+        containing it and all of its derived subclasses, and furnishes
+        the “instances” weak-value dictionary for instance registration.
+    """
+    
+    __slots__ = pytuple('exports', 'weakref') + ('path', 'dotpath')
+    
+    def __new__(cls, *args, path=None, **kwargs):
+        try:
+            instance = super(ExporterBase, cls).__new__(cls, *args, **kwargs)
+        except TypeError:
+            instance = super(ExporterBase, cls).__new__(cls)
+        
+        instance.__exports__ = {}
+        instance.path = path
+        instance.dotpath = path_to_dotpath(instance.path,
+                                           relative_to=cls.prefix)
         
         if instance.dotpath is not None:
             cls.instances[instance.dotpath] = instance
@@ -220,7 +422,7 @@ class Exporter(MutableMapping):
         # b) it’s too useful a method to give it all up. So deal.
         from clu.typology import isstring
         if not isstring(key):
-            raise TypeError("instance access by string keys only")
+            raise TypeError("instance registry access by string keys only")
         return cls.instances[key]
     
     @classmethod
@@ -242,12 +444,23 @@ class Exporter(MutableMapping):
         for modulename in modulenames:
             mods.append(importlib.import_module(modulename))
         
-        modules = dict(zip(modulenames, mods))
-        return modules
+        return dict(zip(modulenames, mods))
     
-    def __init__(self, *args, **kwargs):
-        self.__exports__ = {}
-        
+    @classmethod
+    def nameof(cls, thing):
+        """ Find and return the name of a thing, if that thing
+            should be found to reside in one of the exported modules
+        """
+        return search_modules(thing, *cls.modules().values())[1]
+    
+    @classmethod
+    def moduleof(cls, thing):
+        """ Find and return the module for a thing, if that thing
+            should be found to reside in one of the exported modules
+        """
+        return search_modules(thing, *cls.modules().values())[0]
+    
+    def __init__(self, *args, path=None, **kwargs):
         for arg in args:
             if hasattr(arg, '__exports__'):
                 self.__exports__.update(arg.__exports__)
@@ -346,10 +559,10 @@ class Exporter(MutableMapping):
         # a lambda, try to rename it with either our valid name,
         # or the result of an ID-based search for that lambda:
         if callable(thing) and hasattr(thing, '__name__'):
-            if getattr(thing, '__name__') in (λ, φ):
-                dname = getattr(thing, '__name__')
+            dname = getattr(thing, '__name__')
+            if dname in (λ, φ):
                 if named in (λ, φ):
-                    named = thingname_search(thing)
+                    named = search_for_name(thing)
                 if named is None:
                     raise ExportError(type(self).messages['noname'] % id(thing))
                 thing.__name__ = thing.__qualname__ = named
@@ -423,13 +636,14 @@ class Exporter(MutableMapping):
         """
         return self.dir_function(), self.all_tuple() # OPPOSITE!
     
-    def cache_info(self):
+    @staticmethod
+    def cache_info():
         """ Shortcut to get the CacheInfo namedtuple from the
-            cached internal `thingname_search_by_id(…)` function,
+            cached internal `search_by_id(…)` function,
             which is used in last-resort name lookups made by
             `determine_name(…)` during `export(…)` calls.
         """
-        return thingname_search_by_id.cache_info()
+        return search_by_id.cache_info()
     
     def __iter__(self):
         return iter(self.__exports__.keys())
@@ -484,18 +698,66 @@ class Exporter(MutableMapping):
     
     def __bool__(self):
         return len(self.__exports__) > 0
+    
+    def __dir__(self):
+        return list(filter(lambda name: name not in ('all_appnames',
+                                                     'for_appname',
+                                                     'module_getters',
+                                                     'all_modules'),
+                           super(ExporterBase, self).__dir__()))
+
+class Exporter(ExporterBase, prefix=BASEPATH, appname=PROJECT_NAME):
+    
+    """ A class representing a list of things for a module to export.
+        
+        This class is specifically germane to the CLU project – note
+        that the “prefix” class keyword is used to assign a value from
+        the CLU constants module.
+        
+        Users of CLU who wish to use the Exporter mechanism in their
+        own projects should create a subclass of ExporterBase of their
+        own. Like this one, it need only assign the “prefix” class
+        keyword; it is unnecessary (but OK!) to define further methods,
+        properties, class constants, and whatnot.
+        
+        When writing your subclass, if you •do• choose to add methods
+        or other things, it is imperative that you ensure you aren’t
+        accedentally clobbering anything important from ExporterBase,
+        or you risk UNDEFINED BEHAVIOR!!!! Erm.
+        
+        Also note that all derived subclasses of ExporterBase will
+        automatrically be slotted classes – a “__slots__” attribute
+        will be added to the class dict by the “clu.exporting.Slotted”
+        metaclass, if your subclass doesn’t define one – and so if
+        you desire a class with a working “__dict__” attribute for
+        some reason, you’ll need to specify:
+        
+            __slots__ = tuplize('__dict__')
+        
+        … in your class (or an equivalent).
+    """
+    
+    pass
 
 exporter = Exporter(path=__file__)
 export = exporter.decorator()
 
 export(doctrim)
-export(thingname_search)
+export(itermodule)
+export(moduleids)
+export(itermoduleids)
+export(search_by_id)
+export(search_for_name)
+export(search_for_module)
+export(search_modules)
 export(determine_name)
 export(path_to_dotpath)
 export(predicates_for_types)
 export(sysmods,         name='sysmods',         doc="sysmods() → shortcut for reversed(tuple(frozenset(sys.modules.values()))) …OK? I know. It’s not my finest work, but it works.")
 
 # NO DOCS ALLOWED:
+export(Registry)
+export(ExporterBase)
 export(Exporter)        # hahaaaaa
 
 # Assign the modules’ `__all__` and `__dir__` using the exporter:
